import { DatePipe } from '@angular/common';
import { Component, OnInit } from '@angular/core';
import { ColDef, ICellRendererParams } from 'ag-grid-community';
import { finalize } from 'rxjs/operators';
import { NotificationService } from '../../../services/notification.service';
import { StructureService } from '../../../services/structure.service';
import { AdminStructure } from '../../models/adminStructureList.interface';
import { AdminService } from '../../services/admin.service';
import {
  ButtonCellRendererComponent,
  ButtonCellRendererParams,
} from '../button-cell-renderer/button-cell-renderer.component';
import { CommentCellRendererComponent } from '../comment-cell-renderer/comment-cell-renderer.component';

@Component({
  selector: 'app-admin-structures-list',
  templateUrl: './admin-structures-list.component.html',
  styleUrls: ['../../admin.scss', 'admin-structures-list.component.scss'],
})
export class AdminStructuresListComponent implements OnInit {
  public structuresClaimed: AdminStructure[];
  public structuresInClaim: AdminStructure[];
  public structuresToClaim: AdminStructure[];
  public structuresIncomplete: AdminStructure[];
  public isLoading = true;
  public rowHeight = 48;
  public filter = 'all';
  public defaultColDef: ColDef = {
    editable: true,
    sortable: true,
    flex: 1,
    minWidth: 100,
    filter: true,
    resizable: true,
  };
  public unAttachedColDef: ColDef = {
    ...this.defaultColDef,
    cellStyle: {
      color: '#da6c2e',
    },
  };

  constructor(
    private adminService: AdminService,
    private datePipe: DatePipe,
    private notificationService: NotificationService,
    private structureService: StructureService,
  ) {}

  public filterOptions = [
    { label: 'Toutes les structures', value: 'all' },
    { label: 'Données manquantes', value: 'incomplete' },
    { label: 'En cours de revendication', value: 'inClaim' },
    { label: 'À revendiquer', value: 'toClaim' },
  ];

  public columnDefs: ColDef<AdminStructure>[] = [
    {
      headerName: 'Structure',
      cellRenderer: (params): string => this.renderLink(params.data),
      comparator(_, __, nodeA, nodeB): number {
        return nodeA.data.structureName.toLowerCase() > nodeB.data.structureName.toLowerCase() ? -1 : 1;
      },
      sortable: true,
      flex: 1,
    },
    {
      headerName: 'Date de mise à jour',
      field: 'updatedAt',
      valueFormatter: ({ value }): string => this.datePipe.transform(value, 'mediumDate'),
      cellClass: (params): string => (params.data.isOutdated ? 'red' : ''),
      flex: 1,
      sortable: true,
    },
    {
      headerName: 'Dernière relance par mail',
      field: 'lastUpdateMail',
      valueFormatter: ({ value }): string => this.datePipe.transform(value, 'mediumDate'),
      flex: 1,
      sortable: true,
    },
    {
      headerName: 'Relancer',
      field: 'reminderButton',
      cellRenderer: ButtonCellRendererComponent,
      cellRendererParams: { label: 'Relancer', type: 'reminder' } as ButtonCellRendererParams,
      flex: 1,
      sortable: false,
      onCellClicked: this.handleReminder.bind(this),
    },
    {
      headerName: 'Mettre à jour la date',
      field: 'updateDateButton',
      cellRenderer: ButtonCellRendererComponent,
      cellRendererParams: { label: 'Update', type: 'default' } as ButtonCellRendererParams,
      flex: 1,
      sortable: false,
      onCellClicked: this.handleUpdatedAt.bind(this),
    },
    {
      headerName: 'Commentaire',
      field: 'comment',
      flex: 1,
      editable: true,
      cellEditor: 'agLargeTextCellEditor',
      cellEditorPopup: true,
      cellEditorParams: {
        maxLength: 500,
      },
      sortable: false,
      cellRenderer: CommentCellRendererComponent,
      onCellValueChanged: this.onChangeComment.bind(this),
      autoHeight: true,
    },
  ];

  public gridSections = [];
  public filteredSections = [];

  ngOnInit(): void {
    this.adminService
      .getAllStructureAdmin()
      .pipe(finalize(() => (this.isLoading = false)))
      .subscribe((response) => {
        this.structuresClaimed = response.claimed;
        this.structuresInClaim = response.inClaim;
        this.structuresToClaim = response.toClaim;
        this.structuresIncomplete = response.incomplete;

        this.gridSections = [
          {
            filter: 'incomplete',
            title: 'Structures avec des données manquantes',
            data: this.structuresIncomplete,
            cssClass: 'red',
          },
          {
            filter: 'inClaim',
            title: 'Structures en cours de revendication',
            data: this.structuresInClaim,
            cssClass: '',
          },
          {
            filter: 'toClaim',
            title: 'Structures à revendiquer',
            data: this.structuresToClaim,
            cssClass: '',
          },
          {
            filter: 'claimed',
            title: 'Structures revendiquées',
            data: this.structuresClaimed,
            cssClass: '',
          },
        ];

        this.updateFilteredSections();
      });
  }

  setFilter(filter: string | boolean): void {
    this.filter = filter as string;
    this.updateFilteredSections();
  }

  updateFilteredSections(): void {
    if (this.filter === 'all') {
      this.filteredSections = this.gridSections;
    } else {
      this.filteredSections = this.gridSections.filter((section) => section.filter === this.filter);
    }
  }

  public getGridContext(filterName: string): { context: { gridId: string } } {
    return { context: { gridId: filterName } };
  }

  private renderLink(structure: AdminStructure): string {
    return `<a href="/acteurs?structure=${structure.permalink}" target="_blank">${structure.structureName}</a>`;
  }

  private handleReminder(params: ICellRendererParams): void {
    this.adminService.updateLastUpdateMail(params.data.structureId).subscribe({
      next: () => {
        params.api.getRowNode(String(params.rowIndex)).setDataValue('lastUpdateMail', new Date());
        this.notificationService.showSuccess('Relance effectuée');
      },
      error: (error) => {
        console.error('Error updating last update mail:', error);
        this.notificationService.showError('Erreur: ' + (error.message || 'unknown error'));
      },
    });
  }

  private handleUpdatedAt(params: ICellRendererParams): void {
    this.adminService.touchStructure(params.data.structureId).subscribe({
      next: () => {
        const rowNode = params.api.getRowNode(String(params.rowIndex));
        rowNode.setDataValue('updatedAt', new Date());
        params.data.isOutdated = false;
        params.api.redrawRows({
          rowNodes: [rowNode],
        });
        this.notificationService.showSuccess('Date mise à jour');
      },
      error: (error) => {
        console.error('Error updating updatedAt date:', error);
        this.notificationService.showError('Erreur: ' + (error.message || 'unknown error'));
      },
    });
  }

  public onChangeComment(arg): void {
    const { structureId, ...data } = arg.data;
    const comment = data.comment.slice(0, 500);

    this.structureService.editStructure({ ...data, comment }, structureId).subscribe(() => {
      this.notificationService.showSuccess('Le commentaire a bien été mis à jour.');
    });
  }
}
